local completeList = {}

local function rcvAutocomplete( um )
	local key = um:ReadString()
	local value = ULib.umsgRcv( um )
	completeList[ key ] = value
end
usermessage.Hook( "ulx_autocomplete", rcvAutocomplete )

function ulxListCommands( command, args )
	local targs = string.Trim( args )
	local argv = ULib.splitArgs( targs )

	if #argv == 0 then
		local commandList = {}
		for command, _ in pairs( completeList ) do
			table.insert( commandList, "ulx " .. command )
		end
		return commandList

	elseif #argv == 1 and (targs:len() == 0 or args:sub( -1 ) ~= " ") then -- Take entire arg up to the space
		local commandList = {}
		for command, _ in pairs( completeList ) do
			if command:sub( 1, targs:len() ) == targs then
				table.insert( commandList, "ulx " .. command )
			end
		end
		return commandList

	else
		local command2 = argv[ 1 ]
		local newArgs = " " .. targs:gsub( "^%S+%s*(.*)$", "%1" )
		local info = completeList[ command2 ]
		
		--print( "made it here", command2, newArgs )
		--PrintTable( completeList )
		--PrintTable( info )

		if not info then return end -- Invalid command or they don't have access to it

		local fn
		if info.fn then
			local loc = string.Explode( ".", info.fn )
			fn = _G
			for _, v in ipairs( loc ) do
				fn = fn[ v ]
			end
		end

		if info.id == ulx.ID_ORIGINAL then
			if fn then
				return fn( command .. " " .. command2, newArgs )
			end
		elseif info.id == ulx.ID_HELP then
			return { "ulx " .. command2 .. " " .. info.help }
		elseif info.id == ulx.ID_PLAYER_HELP then
			local targs = string.Trim( newArgs )
			if #argv < 3 and (targs:len() == 0 or args:sub( -1 ) ~= " ") then -- Take entire arg up to the space
				local ret = {}
				local players = ULib.getUsers( targs, true )
				if not players then -- Not valid, grab them all.
					players = player.GetAll()
				end
				for _, player in ipairs( players ) do
					table.insert( ret, string.format( "ulx %s \"%s\"", command2, player:Nick() ) )
				end
				return ret
			else
				return { "ulx " .. command2 .. " " .. info.help }
			end
		end
		return -- TODO: Implement properly
	end
end

ulx.maps = {}
local function rcvMap( um )
	local map = um:ReadString()
	table.insert( ulx.maps, map )
end
usermessage.Hook( "ulx_map", rcvMap )

function mapComplete( command, args )
	local targs = string.Trim( args )
	local mapList = {}
	if targs:len() > 0 then
		for _, map in ipairs( ulx.maps ) do
			if map:sub( 1, targs:len() ) == targs then
				table.insert( mapList, command .. " " .. map )
			end
		end
	else
		for _, map in ipairs( ulx.maps ) do
			table.insert( mapList, command .. " " .. map )
		end
	end

	return mapList
end

function ulx.blindUser( bool, amt )
	if bool then
		local function blind()
			draw.RoundedBox( 0, 0, 0, ScrW(), ScrH(), Color( 255, 255, 255, amt ) )
		end
		hook.Add( "HUDPaint", "ulx_blind", blind )
	else
		hook.Remove( "HUDPaint", "ulx_blind" )
	end
end

local function rcvBlind( um )
	local bool = um:ReadBool()
	local amt = um:ReadShort()
	ulx.blindUser( bool, amt )
end
usermessage.Hook( "ulx_blind", rcvBlind )