--[[
	Title: Fun

	Some fun things for admins
]]

function ulx.cc_slap( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A slapped #T with " .. tostring( argv[2] or 0 ) .. " damage" )
		ULib.slap( v, argv[ 2 ], argv[ 3 ] )
	end
end
ulx.concommand( "slap", ulx.cc_slap, "<user(s)> [<damage>] [<power>] - Slaps target(s) with given damage and power", ULib.ACCESS_OPERATOR, "!slap", _, ulx.ID_PLAYER_HELP )

function ulx.cc_slay( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A slayed #T" )
		v:Kill()
	end
end
ulx.concommand( "slay", ulx.cc_slay, "<user(s)> - slays the specified users", ULib.ACCESS_OPERATOR, "!slay", _, ulx.ID_PLAYER_HELP )

function ulx.cc_sslay( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A sslayed #T" )
		v:KillSilent()
	end
end
ulx.concommand( "sslay", ulx.cc_sslay, "<user(s)> - silently slays the specified users", ULib.ACCESS_OPERATOR, "!sslay", _, ulx.ID_PLAYER_HELP )

function ulx.cc_ignite( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A ignited #T" )
		v:Ignite( 5, 0 )
	end
end
ulx.concommand( "ignite", ulx.cc_ignite, "<user(s)> - ignites the specified users", ULib.ACCESS_OPERATOR, "!ignite", _, ulx.ID_PLAYER_HELP )

function ulx.cc_playsound( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	if not file.Exists( "../sound/" .. argv[ 1 ] ) then
		ULib.tsay( ply, "That sound doesn't exist on the server!" )
		return
	end

	ulx.logServAct( ply, "#A played sound " .. argv[ 1 ] )

	util.PrecacheSound( argv[ 1 ] )

	for _, player in ipairs( player.GetAll() ) do
		ULib.sendLuaFunction( player, "LocalPlayer():EmitSound", argv[ 1 ] )
	end
end
ulx.concommand( "playsound", ulx.cc_playsound, "<sound> - Plays a sound, relative to 'sound' dir.", ULib.ACCESS_OPERATOR )

function ulx.cc_freeze( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A froze #T" )
		v:Lock()
	end
end
ulx.concommand( "freeze", ulx.cc_freeze, "<user(s)> - freezes the specified users", ULib.ACCESS_OPERATOR, "!freeze", _, ulx.ID_PLAYER_HELP )

function ulx.cc_unfreeze( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A unfroze #T" )
		v:UnLock()
	end
end
ulx.concommand( "unfreeze", ulx.cc_unfreeze, "<user(s)> - unfreezes the specified users", ULib.ACCESS_OPERATOR, "!unfreeze", _, ulx.ID_PLAYER_HELP )

function ulx.cc_god( ply, command, argv, args )
	local targets
	if #argv < 1 then
		targets = { ply }
	else
		local err
		targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
		if not targets then
			ULib.tsay( ply, err )
			return
		end
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A granted god mode to #T" )
		v:GodEnable()
	end
end
ulx.concommand( "god", ulx.cc_god, "<user(s)> - Makes player(s) invincible. If no user is specified, gods yourself.", ULib.ACCESS_OPERATOR, "!god", _, ulx.ID_PLAYER_HELP )

function ulx.cc_ungod( ply, command, argv, args )
	local targets
	if #argv < 1 then
		targets = { ply }
	else
		local err
		targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
		if not targets then
			ULib.tsay( ply, err )
			return
		end
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A revoked god mode from #T" )
		v:GodDisable()
	end
end
ulx.concommand( "ungod", ulx.cc_ungod, "<user(s)> - Revokes player'(s) invincibility. If no user is specified, ungods yourself.", ULib.ACCESS_OPERATOR, "!ungod", _, ulx.ID_PLAYER_HELP )

function ulx.cc_noclip( ply, command, argv, args )
	local targets
	if #argv < 1 then
		targets = { ply }
	else
		local err
		targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
		if not targets then
			ULib.tsay( ply, err )
			return
		end
	end

	for _, v in ipairs( targets ) do
		if v:GetMoveType() == MOVETYPE_WALK then
			v:SetMoveType( MOVETYPE_NOCLIP )
		elseif v:GetMoveType() == MOVETYPE_NOCLIP then
			v:SetMoveType( MOVETYPE_WALK )
		end -- Ignore if they're an observer
	end
end
ulx.concommand( "noclip", ulx.cc_noclip, "<user(s)> - Toggles noclip for player(s). If no user is specified, noclips yourself.", ULib.ACCESS_OPERATOR, "!noclip", _, ulx.ID_PLAYER_HELP )

function ulx.cc_hp( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end
	
	local hp = tonumber( argv[ 2 ] )
	if not hp then 
		ULib.tsay( userid, "\"" .. argv[ 2 ] .. "\" is an invalid health!" )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A set #T's hp to " .. hp )
		v:SetHealth( hp )
	end
end
ulx.concommand( "hp", ulx.cc_hp, "<user(s)> - sets the hp for the specified users", ULib.ACCESS_OPERATOR, "!hp", _, ulx.ID_PLAYER_HELP )

function ulx.cc_cloak( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end
	
	local amt = tonumber( argv[ 2 ] ) or 0

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A cloaked #T by amount " .. amt )
		v:SetKeyValue( "rendermode", "1" )
		v:Fire( "alpha", tostring( amt ), 0 )
		v:DrawViewModel( false )
		v:DrawWorldModel( false )
	end
end
ulx.concommand( "cloak", ulx.cc_cloak, "<user(s)> [<amt 0-255>] - Cloaks user(s) by amount.", ULib.ACCESS_OPERATOR, "!cloak", _, ulx.ID_PLAYER_HELP )

function ulx.cc_uncloak( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A uncloaked #T" )
		v:SetKeyValue( "rendermode", "0" )		
		v:Fire( "alpha", tostring( amt ), 255 )
		v:DrawViewModel( true )
		v:DrawWorldModel( true )
	end
end
ulx.concommand( "uncloak", ulx.cc_uncloak, "<user(s)> - Uncloaks user(s).", ULib.ACCESS_OPERATOR, "!uncloak", _, ulx.ID_PLAYER_HELP )

function ulx.cc_blind( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end
	
	local amt = tonumber( argv[ 2 ] ) or 255

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A blinded #T by amount " .. amt )
		umsg.Start( "ulx_blind", v )
			umsg.Bool( true )
			umsg.Short( amt )
		umsg.End()
	end
end
ulx.concommand( "blind", ulx.cc_blind, "<user(s)> [<amt 0-255>] - Blinds user(s) by amount.", ULib.ACCESS_OPERATOR, "!blind", _, ulx.ID_PLAYER_HELP )

function ulx.cc_unblind( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A unblinded #T" )
		umsg.Start( "ulx_blind", v )
			umsg.Bool( false )
			umsg.Short( 0 )
		umsg.End()
	end
end
ulx.concommand( "unblind", ulx.cc_unblind, "<user(s)> - Unblinds user(s).", ULib.ACCESS_OPERATOR, "!unblind", _, ulx.ID_PLAYER_HELP )

local jailed = {}

function ulx.cc_jail( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	local time = tonumber( argv[ 2 ] ) or 0

	for _, v in ipairs( targets ) do
		if time > 0 then
			ulx.logUserAct( ply, v, "#A jailed #T for " .. time .. " seconds" )
		else
			ulx.logUserAct( ply, v, "#A jailed #T" )
		end

		local pos = v:GetPos()

		local top = ents.Create( "prop_dynamic_override" )
		top:SetKeyValue( "solid", "6" )
		top:SetModel( "models/props_c17/fence02b.mdl" )
		top:SetPos( Vector( pos.x + 35, pos.y, pos.z + 60 ) )
		top:SetAngles( Angle( 0, 0, 0 ) )
		top:Spawn()

		local bottom = ents.Create( "prop_dynamic_override" )
		bottom:SetKeyValue( "solid", "6" )
		bottom:SetModel( "models/props_c17/fence02b.mdl" )
		bottom:SetPos( Vector( pos.x - 35, pos.y, pos.z + 60 ) )
		bottom:SetAngles( Angle( 0, 0, 0 ) )
		bottom:Spawn()

		local right = ents.Create( "prop_dynamic_override" )
		right:SetKeyValue( "solid", "6" )
		right:SetModel( "models/props_c17/fence02b.mdl" )
		right:SetPos( Vector( pos.x, pos.y + 35, pos.z + 60 ) )
		right:SetAngles( Angle( 0, 90, 0 ) )
		right:Spawn()

		local left = ents.Create( "prop_dynamic_override" )
		left:SetKeyValue( "solid", "6" )
		left:SetModel( "models/props_c17/fence02b.mdl" )
		left:SetPos( Vector( pos.x, pos.y - 35, pos.z + 60 ) )
		left:SetAngles( Angle( 0, 90, 0 ) )
		left:Spawn()

		local up = ents.Create( "prop_dynamic_override" )
		up:SetKeyValue( "solid", "6" )
		up:SetModel( "models/props_c17/fence01b.mdl" )
		up:SetPos( Vector( pos.x, pos.y, pos.z + 110 ) )
		up:SetAngles( Angle( 90, 0, 0 ) )
		up:Spawn()

		local down = ents.Create( "prop_dynamic_override" )
		down:SetKeyValue( "solid", "6" )
		down:SetModel( "models/props_c17/fence01b.mdl" )
		down:SetPos( Vector( pos.x, pos.y, pos.z - 5 ) )
		down:SetAngles( Angle( 90, 0, 0 ) )
		down:Spawn()
		
		local function unjail()
			ulx.disallowDel[ top ] = nil
			ulx.disallowDel[ bottom ] = nil
			ulx.disallowDel[ right ] = nil
			ulx.disallowDel[ left ] = nil
			ulx.disallowDel[ up ] = nil
			ulx.disallowDel[ down ] = nil
			ulx.disallowMove[ top ] = nil
			ulx.disallowMove[ bottom ] = nil
			ulx.disallowMove[ right ] = nil
			ulx.disallowMove[ left ] = nil
			ulx.disallowMove[ up ] = nil
			ulx.disallowMove[ down ] = nil
			ulx.disallowNoclip[ v ] = nil

			top:Remove()
			bottom:Remove()
			right:Remove()
			left:Remove()
			up:Remove()
			down:Remove()
			
			jailed[ v ] = nil
		end
		if time > 0 then
			timer.Simple( time, unjail )
		end

		jailed[ v ] = { pos=pos, unjail=unjail }
		ulx.disallowDel[ top ] = true
		ulx.disallowDel[ bottom ] = true
		ulx.disallowDel[ right ] = true
		ulx.disallowDel[ left ] = true
		ulx.disallowDel[ up ] = true
		ulx.disallowDel[ down ] = true
		ulx.disallowMove[ top ] = true
		ulx.disallowMove[ bottom ] = true
		ulx.disallowMove[ right ] = true
		ulx.disallowMove[ left ] = true
		ulx.disallowMove[ up ] = true
		ulx.disallowMove[ down ] = true
		ulx.disallowNoclip[ v ] = true
	end
end
ulx.concommand( "jail", ulx.cc_jail, "<user(s)> [<time>] - Jails user(s) for specified seconds (forever if not specified).", ULib.ACCESS_OPERATOR, "!jail", _, ulx.ID_PLAYER_HELP )

function ulx.cc_unjail( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		if jailed[ v ] then
			ulx.logUserAct( ply, v, "#A unjailed #T" )
			jailed[ v ].unjail()
		end
	end
end
ulx.concommand( "unjail", ulx.cc_unjail, "<user(s)> - Unjails user(s).", ULib.ACCESS_OPERATOR, "!unjail", _, ulx.ID_PLAYER_HELP )

local function jailCheck( ply )
	if jailed[ ply ] then
		ply:SetPos( jailed[ ply ].pos )
	end
end
hook.Add( "PlayerSpawn", "ULXJailCheck", jailCheck )

local ragdolls = {} -- Holds our ragdolls

function ulx.cc_ragdoll( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A ragdolled #T" )
		local ragdoll = ents.Create( "prop_ragdoll" )
		ragdoll:SetPos( v:GetPos() )
		ragdoll:SetModel( v:GetModel() )
		ragdoll:Spawn()
		ragdoll:Activate()
		
		v:Spectate( OBS_MODE_CHASE )
		v:SpectateEntity( ragdoll )
		v:StripWeapons() -- Otherwise they can still use the weapons.
		
		ragdolls[ v ] = ragdoll
		ulx.disallowDel[ ragdoll ] = true
	end
end
ulx.concommand( "ragdoll", ulx.cc_ragdoll, "<user(s)> - ragdolls the specified users", ULib.ACCESS_OPERATOR, "!ragdoll", _, ulx.ID_PLAYER_HELP )

function ulx.cc_unragdoll( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		local ragdoll = ragdolls[ v ]
		if ragdoll then -- Only if they're ragdolled...
			ulx.logUserAct( ply, v, "#A unragdolled #T" )
			local pos = ragdoll:GetPos()
			pos.z = pos.z + 10 -- So they don't end up in the ground
			ulx.disallowDel[ ragdoll ] = nil
			ragdoll:Remove()
			v:Spawn() -- Take them out of observer mode, and give them weapons back.
			v:SetPos( pos )
		end

		ragdolls[ v ] = nil
	end
end
ulx.concommand( "unragdoll", ulx.cc_unragdoll, "<user(s)> - unragdolls the specified users", ULib.ACCESS_OPERATOR, "!unragdoll", _, ulx.ID_PLAYER_HELP )

local function checkRagdolls( ent )
	for ply, ragdoll in pairs( ragdolls ) do
		if ragdoll == ent then
			local pos = ragdoll:GetPos()
			pos.z = pos.z + 10 -- So they don't end up in the ground
			ulx.disallowDel[ ragdoll ] = nil
			ragdoll:Remove()
			ply:Spawn() -- Take them out of observer mode, and give them weapons back.
			ply:SetPos( pos )
		end
	end
end
hook.Add( "EntityRemoved", "ulx_ragdoll_check", checkRagdolls )

local gimps = {} -- Holds gimp info
local gimpSays = {} -- Holds gimp says
local ID_GIMP = 1
local ID_MUTE = 2

function ulx.cc_gimp( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A gimped #T" )
		gimps[ v:UniqueID() ] = ID_GIMP
	end
end
ulx.concommand( "gimp", ulx.cc_gimp, "<user> - Gimps a player (can't talk normally)", ULib.ACCESS_OPERATOR, "!gimp", _, ulx.ID_PLAYER_HELP )

function ulx.cc_ungimp( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A ungimped #T" )
		gimps[ v:UniqueID() ] = nil
	end
end
ulx.concommand( "ungimp", ulx.cc_ungimp, "<user> - Ungimps a player", ULib.ACCESS_OPERATOR, "!ungimp", _, ulx.ID_PLAYER_HELP )

function ulx.cc_mute( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A muted #T" )
		gimps[ v:UniqueID() ] = ID_MUTE
	end
end
ulx.concommand( "mute", ulx.cc_mute, "<user> - Mutes a player (can't talk)", ULib.ACCESS_OPERATOR, "!mute", _, ulx.ID_PLAYER_HELP )

function ulx.cc_unmute( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local targets, err = ULib.getUsers( argv[ 1 ], _, true, ply ) -- Enable keywords
	if not targets then
		ULib.tsay( ply, err )
		return
	end

	for _, v in ipairs( targets ) do
		ulx.logUserAct( ply, v, "#A unmuted #T" )
		gimps[ v:UniqueID() ] = nil
	end
end
ulx.concommand( "unmute", ulx.cc_unmute, "<user> - Unmutes a player", ULib.ACCESS_OPERATOR, "!unmute", _, ulx.ID_PLAYER_HELP )

function ulx.cc_addGimpSay( ply, command, argv, args )
	local say = ULib.stripQuotes( args )
	table.insert( gimpSays, say )
end
ulx.concommand( "addGimpSay", ulx.cc_addGimpSay, "", ULib.ACCESS_NONE ) -- Can only add from configs


---------------------
--      Hooks      --
---------------------


local function gimpCheck( ply, strText )
	local id = ply:UniqueID()
	if not gimps[ id ] then return nil end
	if gimps[ id ] == ID_MUTE then return "" end
	if gimps[ id ] == ID_GIMP then
		if #gimpSays < 1 then return nil end
		return gimpSays[ math.random( #gimpSays ) ]
	end
end
hook.Add( "PlayerSay", "ULXGimpCheck", gimpCheck )