function ulx.cc_who( ply, command, argv, args )
	ULib.console( ply, "Userid   Name   Level   Immune" )
	local players = player.GetAll()
	for _, player in ipairs( players ) do
		local text = string.format( "%i      %s", player:EntIndex(), player:Nick() )
		if player:IsUserGroup( ULib.ACCESS_SUPERADMIN ) then
			text = text .. "   Superadmin"
		elseif player:IsUserGroup( ULib.ACCESS_ADMIN ) then
			text = text .. "   Admin"
		elseif player:IsUserGroup( ULib.ACCESS_OPERATOR ) then
			text = text .. "   Operator"
		else
			text = text .. "   Guest"
		end
		
		if player:IsUserGroup( ULib.ACCESS_IMMUNITY ) then
			text = text .. "   yes"
		else
			text = text .. "   no"
		end
		ULib.console( ply, text )
	end
end
ulx.concommand( "who", ulx.cc_who, "- Shows all connected players ids and access.", ULib.ACCESS_ALL )

function ulx.cc_psay( ply, command, argv, args )
	if #argv < 2 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local target, err = ULib.getUser( argv[ 1 ], _, ply )
	if not target then
		ULib.tsay( ply, err )
		return
	end
	
	if target == ply then
		ULib.tsay( ply, "Why are you talking to yourself?" )
		return
	end
	
	local str = args:gsub( "^%S+%s+(.*)$", "%1" ) -- Remove first arg
	local message = string.format( "%s to %s: %s", ply:Nick(), target:Nick(), str )

	ULib.tsay( target, message )
	ULib.tsay( ply, message )

	if util.tobool( GetConVarNumber( "ulx_logChat" ) ) then
		ulx.logString( message )
	end
end
ulx.concommand( "psay", ulx.cc_psay, "<user> <text> - Sends a private message to the user.", ULib.ACCESS_ALL, "!p", true, ulx.ID_PLAYER_HELP )

function ulx.cc_asay( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

       	local message = string.format( "%s to admins: %s", ply:Nick(), args )
	local players = player.GetAll()
	for _, player in ipairs( players ) do
		if player:IsUserGroup( ULib.ACCESS_OPERATOR ) and player ~= ply then
			ULib.tsay( player, message )
		end
	end
	ULib.tsay( ply, message ) -- Make sure they get it too.

	if util.tobool( GetConVarNumber( "ulx_logChat" ) ) then
		ulx.logString( message )
	end
end
ulx.concommand( "asay", ulx.cc_asay, "<text> - Sends a message to currently connected admins.", ULib.ACCESS_ALL, "@", true, ulx.ID_HELP )

function ulx.cc_tsay( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	ULib.tsay( _, args )

	if util.tobool( GetConVarNumber( "ulx_logChat" ) ) then
		ulx.logString( "(tsay) " .. args )
	end
end
ulx.concommand( "tsay", ulx.cc_tsay, "<text> - Sends a message to everyone.", ULib.ACCESS_OPERATOR, "@@", true, ulx.ID_HELP )

function ulx.cc_csay( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	ULib.csay( _, args )

	if util.tobool( GetConVarNumber( "ulx_logChat" ) ) then
		ulx.logString( "(csay) " .. args )
	end
end
ulx.concommand( "csay", ulx.cc_csay, "<text> - Sends a message to everyone in the middle of the screen.", ULib.ACCESS_OPERATOR, "@@@", true, ulx.ID_HELP )

function ulx.cc_rcon( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	game.ConsoleCommand( args .. "\n" )
	ulx.logServAct( ply, "#A ran rcon command: " .. args, true )
end
ulx.concommand( "rcon", ulx.cc_rcon, "<command> - Send the specified command to the server console.", ULib.ACCESS_SUPERADMIN, "!rcon", true, ulx.ID_HELP )

function ulx.cc_luaRun( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	game.ConsoleCommand( "lua_run " .. args .. "\n" )
	ulx.logServAct( ply, "#A ran lua_run: " .. args, true )
end
ulx.concommand( "lua_run", ulx.cc_luaRun, "<command> - Feeds the server a lua command.", ULib.ACCESS_SUPERADMIN, _, _, ulx.ID_HELP )

function ulx.cc_exec( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

        if file.Exists( "../cfg/" .. argv[ 1 ] ) or file.Exists( "../cfg/" .. argv[ 1 ] .. ".cfg" ) then
		game.ConsoleCommand( "exec " .. argv[ 1 ] .. "\n" )
		ulx.logServAct( ply, "#A executed " .. argv[ 1 ], true )
        else
        	ULib.tsay( ply, "That config does not exist!" )
        end
end
ulx.concommand( "exec", ulx.cc_exec, "<file> - Executes a file.", ULib.ACCESS_ADMIN, _, _, ulx.ID_HELP )

function ulx.cc_map( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local map = argv[ 1 ]
	if string.sub( map, -4 ) == ".bsp" then
		map = string.sub( map, 1, -5 ) -- Take off the .bsp
	end

        if file.Exists( "../maps/" .. map .. ".bsp" ) then
		game.ConsoleCommand( "changelevel " .. map .. "\n" )
		ulx.logServAct( ply, "#A changed map to " .. map )
        else
        	ULib.tsay( ply, "That map does not exist!" )
        end
end
ulx.concommand( "map", ulx.cc_map, "<map> - Changes the map.", ULib.ACCESS_ADMIN, "!map", _, ulx.ID_ORIGINAL, "mapComplete" )

function ulx.cc_kick( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local target, err = ULib.getUser( argv[ 1 ], _, ply )
	if not target then
		ULib.tsay( ply, err )
		return
	end

	if argv[ 2 ] then
		ulx.logUserAct( ply, target, "#A kicked #T (" .. argv[ 2 ] .. ")" )
	else
		ulx.logUserAct( ply, target, "#A kicked #T" )
	end

	ULib.kick( target, argv[ 2 ] )
end
ulx.concommand( "kick", ulx.cc_kick, "<user> [<reason>] - Kicks a user with the given reason", ULib.ACCESS_ADMIN, "!kick", _, ulx.ID_PLAYER_HELP )

function ulx.cc_ban( ply, command, argv, args )
	if #argv < 2 then
		ULib.tsay( userid, ulx.LOW_ARGS )
		return
	end

	local target, err = ULib.getUser( argv[ 1 ], _, ply )
	if not target then
		ULib.tsay( ply, err )
		return
	end
	
	local bantime = tonumber( argv[ 2 ] )
	if not bantime then ULib.tsay( ply, "Invalid ban time!" ) return end

	local time = "for " .. argv[ 2 ] .. " minute(s)"
	if bantime == 0 then time = "permanently" end
	ulx.logUserAct( ply, target, "#A banned #T " .. time )

	ULib.ban( target, bantime )
end
ulx.concommand( "ban", ulx.cc_ban, "<user> <time> - Bans a user for time, use 0 time for permaban", ULib.ACCESS_ADMIN, "!ban", _, ulx.ID_PLAYER_HELP )

function ulx.cc_spectate( ply, command, argv, args )
	if #argv < 1 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local target, err = ULib.getUser( argv[ 1 ], _, ply )
	if not target then
		ULib.tsay( ply, err )
		return
	end
	
	if target == ply then
		ULib.tsay( ply, "You can't spectate yourself!" )
		return
	end

	local pos = ply:GetPos()
	local ang = ply:GetAngles()
	local uid = ply:UniqueID()
	local function unspectate( player, key )
		if ply ~= player then return end -- Not the person we want
		if player:UniqueID() ~= uid then hook.Remove( "KeyPress", "ulx_unspectate_" .. ply:EntIndex() ) return end -- Different person
		if key ~= IN_FORWARD and key ~= IN_BACK and key ~= IN_MOVELEFT and key ~= IN_MOVERIGHT then return end -- Not a key we're interested in

		player:Spawn() -- Get out of spectate
		player:SetPos( pos )
		player:SetAngles( ang )
		ulx.logUserAct( player, target, "#A stopped spectating #T", true )
		hook.Remove( "KeyPress", "ulx_unspectate_" .. ply:EntIndex() )
	end
	hook.Add( "KeyPress", "ulx_unspectate_" .. ply:EntIndex(), unspectate )

	ulx.logUserAct( ply, target, "#A began spectating #T", true )
	ply:Spectate( OBS_MODE_IN_EYE )
	ply:SpectateEntity( target )
	ply:StripWeapons()
end
ulx.concommand( "spectate", ulx.cc_spectate, "<user> - Spectates specified user. Move forward to get out of this mode.", ULib.ACCESS_ADMIN, "!spectate", true, ulx.ID_PLAYER_HELP )

function ulx.cc_ent( ply, command, argv, args )
	if #argv < 2 then
		ULib.tsay( userid, ulx.LOW_ARGS )
		return
	end

	local class = argv[ 1 ]
	table.remove( argv, 1 ) -- We already have the first now
	newEnt = ents.Create( class )

	local trace = ply:GetEyeTrace()
	local vector = trace.HitPos
	vector.z = vector.z + 20

	newEnt:SetPos( vector ) -- Note that the position can be overridden by the users's flags

	for i, v in ipairs( argv ) do -- Loop through them
		local key = string.sub( v, 1, string.find( v, ":" ) - 1 )
		local value = string.sub( v, string.find( v, ":" ) + 1 )
		newEnt:SetKeyValue( key, value )
	end

	newEnt:Spawn()
	newEnt:Activate()

	ulx.logServAct( ply, target, "#A created ent \"" .. args .. "\"" )
end
ulx.concommand( "ent", ulx.cc_ent, "<classname> <flag>:<value> .. - spawn an ent, separate flag and value with : (IE: ulx ent prop_ragdoll model:models/alyx.mdl)", ULib.ACCESS_SUPERADMIN, _, _, ulx.ID_HELP )

function ulx.cc_cexec( ply, command, argv, args )
	if #argv < 2 then
		ULib.tsay( ply, ulx.LOW_ARGS )
		return
	end

	local target, err = ULib.getUser( argv[ 1 ], _, ply )
	if not target then
		ULib.tsay( ply, err )
		return
	end

	local cmd = string.gsub( args, "^%S+%s+(.*)$", "%1" )
	target:ConCommand( cmd )
	ulx.logUserAct( ply, target, "#A ran \"" .. cmd .. "\" on #T", true )
end
ulx.concommand( "cexec", ulx.cc_cexec, "<user(s)> <command> - Runs a command on the user's console.", ULib.ACCESS_SUPERADMIN, "!cexec", true, ulx.ID_PLAYER_HELP )

function ulx.cc_thetime( ply, command, argv, args )
	ULib.tsay( _, os.date( "The time is now %I:%M %p." ) )
end
ulx.concommand( "thetime", ulx.cc_thetime, " - Shows you the server time.", ULib.ACCESS_ALL, "!thetime", _, ulx.ID_HELP )

function ulx.cc_menu( ply, command, argv, args )
	ULib.sendLuaFunction( ply, "ulx.showMainMenu" )
end
ulx.concommand( "menu", ulx.cc_menu, " - Shows you the main menu.", ULib.ACCESS_ALL, "!menu", _, ulx.ID_HELP )

function ulx.cc_clientmenu( ply, command, argv, args )
	ULib.sendLuaFunction( ply, "ulx.showClientMenu" )
end
ulx.concommand( "clientmenu", ulx.cc_clientmenu, " - Shows you the client menu.", ULib.ACCESS_OPERATOR, "!clientmenu", _, ulx.ID_HELP )

function ulx.cc_adminmenu( ply, command, argv, args )
	ULib.sendLuaFunction( ply, "ulx.showAdminMenu" )
end
ulx.concommand( "adminmenu", ulx.cc_adminmenu, " - Shows you the admin menu.", ULib.ACCESS_OPERATOR, "!adminmenu", _, ulx.ID_HELP )

local cadverts = { count=0 }
local tadverts = { count=0 }

function ulx.cc_addAdvert( ply, command, argv, args )
	if #argv < 2 then return end -- Invalid
	
	local message = argv[ 1 ] or ""
	local rpt = tonumber( argv[ 2 ] ) or 360
	local group = argv[ 3 ]

	message = string.gsub( message, "%%curmap%%", game.GetMap() )
	message = string.gsub( message, "%%host%%", GetConVarString( "hostname" ) )
	message = string.gsub( message, "%%ulx_version%%", string.format( "%.02f", ulx.version ) )
	
	tadverts.count = tadverts.count + 1

	if not group then
		timer.Create( "TSay" .. tadverts.count, rpt, 0, ULib.tsay, _, message )
	else
		if not tadverts[ group ] then
			tadverts[ group ] = { last=0 }
			local timerName = "TSay" .. tadverts.count
			local function doAdvert()
				local id = tadverts[ group ].last + 1
				tadverts[ group ].last = math.fmod( tadverts[ group ].last + 1, #tadverts[ group ] ) -- Reset the last

				ULib.tsay( _, tadverts[ group ][ id ].message )
				timer.Adjust( timerName, tadverts[ group ][ tadverts[ group ].last + 1 ].rpt, 0, doAdvert )
				timer.Start( timerName )
			end
			timer.Create( timerName, rpt, 0, doAdvert )
		end
		table.insert( tadverts[ group ], { rpt=rpt, message=message } )
	end
end
ulx.concommand( "addAdvert", ulx.cc_addAdvert, "", ULib.ACCESS_NONE ) -- Can only add from configs

function ulx.cc_addCsayAdvert( ply, command, argv, args )
	if #argv < 6 then return end -- Invalid

	local message = argv[ 1 ] or ""
	local r = tonumber( argv[ 2 ] ) or 255
	local g = tonumber( argv[ 3 ] ) or 255
	local b = tonumber( argv[ 4 ] ) or 255
	local rpt = tonumber( argv[ 5 ] ) or 300
	local len = tonumber( argv[ 6 ] ) or 10
	local group = argv[ 7 ]

	message = string.gsub( message, "%%curmap%%", game.GetMap() )
	message = string.gsub( message, "%%host%%", GetConVarString( "hostname" ) )
	message = string.gsub( message, "%%ulx_version%%", string.format( "%.02f", ulx.version ) )
	
	cadverts.count = cadverts.count + 1

	if not group then
		timer.Create( "CSay" .. cadverts.count, rpt, 0, ULib.csay, _, message, Color( r, g, b, 255 ), len )
	else
		if not cadverts[ group ] then
			cadverts[ group ] = { last=0 }
			local timerName = "CSay" .. cadverts.count
			local function doAdvert()
				local id = cadverts[ group ].last + 1
				cadverts[ group ].last = math.fmod( cadverts[ group ].last + 1, #cadverts[ group ] ) -- Reset the last

				ULib.csay( _, cadverts[ group ][ id ].message, cadverts[ group ][ id ].color, cadverts[ group ][ id ].len )
				timer.Adjust( timerName, cadverts[ group ][ cadverts[ group ].last + 1 ].rpt, 0, doAdvert )
				timer.Start( timerName )
			end
			timer.Create( timerName, rpt, 0, doAdvert )
		end
		table.insert( cadverts[ group ], { rpt=rpt, message=message, color=Color( r, g, b, 255 ), len=len } )
	end
end
ulx.concommand( "addCsayAdvert", ulx.cc_addCsayAdvert, "", ULib.ACCESS_NONE ) -- Can only add from configs

--------------------
--     Hooks      --
--------------------
function checkMinge( name )
	if util.tobool( GetConVarNumber( "ulx_mingekick" ) ) then
		if name == "MingeBag" or string.sub( name, 4 ) == "MingeBag" then -- Catches it with numbers in front as well
			print( "caught" )
			game.ConsoleCommand( "kick \"" .. name .. "\"\n" )
			ULib.tsay( _, "[ULX] Auto-minge kick" )
		end
	end
end
hook.Add( "PlayerConnect", "ULXMingeCheck", checkMinge )
ulx.convar( "mingekick", "0", "<0/1> - Turns off/on auto-minge bag kick", ULib.ACCESS_ADMIN )

function showWelcome( ply )
	local message = GetConVarString( "ulx_welcomemessage" )
	if not message or message == "" then return end

	message = string.gsub( message, "%%curmap%%", game.GetMap() )
	message = string.gsub( message, "%%host%%", GetConVarString( "hostname" ) )
	message = string.gsub( message, "%%ulx_version%%", string.format( "%.02f", ulx.version ) )

	ULib.tsay( ply, message )
end
hook.Add( "PlayerInitialSpawn", "ULXWelcome", showWelcome )
ulx.convar( "welcomemessage", "", "<msg> - This is shown to players on join", ULib.ACCESS_ADMIN )